-- Released under GPL v3
-------------------------------------------------------------------------------------------------
include("RSD_Utils");
include("RSD_Fog");

local opened = false
local hideFog = false;
local revealMap = false;
local isPreviewing = false;
local restoreFogAfterMapClose = false;
local restoreViewAfterMapClose = false;

local db = Modding.OpenUserData("Reseed!", 1);
local options = { speed = 1, showYields = false, showUnknownResources };
local updateHandler = nil

options.speed =					tonumber(db.GetValue("Speed") or "3")
options.show =					(db.GetValue("Show")					== "true")
options.showYields =			(db.GetValue("ShowYields")				== "true")
options.showImprovements =		(db.GetValue("ShowImprovements")		== "true")
options.showUnknownResources =	(db.GetValue("ShowUnknownResources")	== "true")
LuaEvents.RSD_ShareOptions(options)

-------------------------------------------------------------------------------------------------
local function OnSaveOptions()
	db.SetValue("Speed",					options.speed);
	db.SetValue("ShowYields",				options.showYields);
	db.SetValue("ShowImprovements",			options.showImprovements);
	db.SetValue("ShowUnknownResources",		options.showUnknownResources);
end
LuaEvents.RSD_SaveOptions.Add(OnSaveOptions)

-------------------------------------------------------------------------------------------------
function OnShowNewMapProgress()
	db.SetValue("Show", "true");
	UnhookFog();
	hideFog = false;
end
LuaEvents.RSD_ShowNewMapProgress.Add(OnShowNewMapProgress)

-------------------------------------------------------------------------------------------------
function OnBeginRoutine(_updateHandler, endCallback)
	assert(_updateHandler)
	updateHandler = function()
		local result = _updateHandler()
		if  result ~= "yield" then 
			updateHandler = nil
			UIManager:SetUICursor(0);
			ContextPtr:ClearUpdate() 
			print("stopped coroutine", result)
			if endCallback then endCallback() end
		end
	end

	ContextPtr:SetUpdate(updateHandler);
	UIManager:SetUICursor(1);
end
LuaEvents.RSD_BeginRoutine.Add(OnBeginRoutine)

-------------------------------------------------------------------------------------------------
function OnSuspendRoutine()
	ContextPtr:ClearUpdate();
end
LuaEvents.RSD_SuspendRoutine.Add(OnSuspendRoutine)

-------------------------------------------------------------------------------------------------
function OnResumeRoutine()
	ContextPtr:SetUpdate(updateHandler);
end
LuaEvents.RSD_ResumeRoutine.Add(OnResumeRoutine)

-------------------------------------------------------------------------------------------------
local function OnRoutineError()
	db.SetValue("Show", "false");
end
LuaEvents.RSD_RoutineError.Add(OnRoutineError)

-------------------------------------------------------------------------------------------------
local function DoHideFog()
	if hideFog then return end
	print("Hiding fog");
	LuaEvents.RSD_HiddenFog()
	HideFog();
	hideFog = true
end

-------------------------------------------------------------------------------------------------
local function DoRestoreFog()
	if not hideFog then return end
	print("Restoring fog");
	RestoreFog();
	hideFog = false

	local pPlayer = Players[Game.GetActivePlayer()];
	local plot = pPlayer:GetStartingPlot();
	UI.LookAt(plot, 0);
end
LuaEvents.RSD_RestoreFog.Add(DoRestoreFog)

-------------------------------------------------------------------------------------------------
local function CheckFogVisibility()
	if revealMap or isPreviewing then
		DoHideFog()
	else
		DoRestoreFog()
	end
end

-------------------------------------------------------------------------------------------------
local function CheckOverlayVisibility()
	if revealMap or isPreviewing then
		LuaEvents.RSD_ShowOverlay(revealMap)
	else
		LuaEvents.RSD_HideOverlay()
	end
end

-------------------------------------------------------------------------------------------------
function OnToggleRevealMap()
	revealMap = not revealMap
	CheckFogVisibility()
	CheckOverlayVisibility()
	LuaEvents.RSD_RevealMapChanged(revealMap)
end
LuaEvents.RSD_ToggleRevealMap.Add(OnToggleRevealMap)

-------------------------------------------------------------------------------------------------
local function OnHidePreview()
	print("OnHidePreview")
    Events.SystemUpdateUI.CallImmediate(SystemUpdateUIType.BulkShowUI);

	if restoreViewAfterMapClose then
		restoreViewAfterMapClose = false
		print("ToggleStrategicView")
		ToggleStrategicView()
	end

	isPreviewing = false
	CheckOverlayVisibility()
	CheckFogVisibility()
end
LuaEvents.RSD_HidePreview.Add(OnHidePreview)

-------------------------------------------------------------------------------------------------
local function OnShowPreview()
	print("RESEED: Loader.OnShowPreview")
    Events.SystemUpdateUI.CallImmediate(SystemUpdateUIType.BulkHideUI);

	if InStrategicView() then
		restoreViewAfterMapClose = false
	else
		restoreViewAfterMapClose = true
		ToggleStrategicView()
	end

	isPreviewing = true
	CheckOverlayVisibility()
	CheckFogVisibility()
end
LuaEvents.RSD_ShowPreview.Add(OnShowPreview)

--------------------------------------------------------------
local function OnClose()
	opened = false
	updateHandler = nil
	UIManager:SetUICursor(0);
	ContextPtr:ClearUpdate()
	LuaEvents.RSD_HidePreview()
	if revealMap then 
		LuaEvents.RSD_ToggleRevealMap()
	end
end
LuaEvents.RSD_Close.Add(OnClose)

-------------------------------------------------------------------------------------------------
local function Show()
	opened = true
	LuaEvents.RSD_Show()
end

-------------------------------------------------------------------------------------------------
function InputHandler(uiMsg, wParam, lParam)
	if uiMsg == KeyEvents.KeyDown then
		if wParam == Keys.R and UIManager:GetControl() and UIManager:GetShift() then
			if opened then 
				LuaEvents.RSD_Close()
			else
				Show()
			end
			return true;

		--[[elseif wParam == Keys.VK_RETURN then
			LuaEvents.RSD_PreviewNewMap()
			return true;]]

		elseif wParam == Keys.VK_ESCAPE then
			if isPreviewing then
				print("hide map")
				LuaEvents.RSD_HidePreview()
				return true;
			elseif opened then
				print("close")
				LuaEvents.RSD_Close()
				return true;
			end
		end
	end
end
ContextPtr:SetInputHandler(InputHandler);

-------------------------------------------------------------------------------------------------
local function OnAdditionalInformationDropdownGatherEntries(additionalEntries)
	table.insert(additionalEntries, 
	{
		text=L("TXT_KEY_RSD_HEADER"), 
		call=Show,
	});
end
LuaEvents.AdditionalInformationDropdownGatherEntries.Add(OnAdditionalInformationDropdownGatherEntries)
LuaEvents.RequestRefreshAdditionalInformationDropdownEntries()

-------------------------------------------------------------------------------------------------
--DeleteFiles("__reseed");
db.SetValue("Show", "false");
if options.show then Events.LoadScreenClose.Add(Show) end

print("'Reseed!'... loaded");
print("v11.06")
