-- Released under GPL v3
--------------------------------------------------------------
include("RSD_Utils")
local mapRoutine = nil;
local shown = false
local step = 0


-------------------------------------------------------------------------------------------------
function GetUnresetableItems()
	local wonderClasses = {}
	for row in GameInfo.BuildingClasses{MaxGlobalInstances = 1} do
		wonderClasses[row.Type] = true
	end

	local wonders = {}
	for row in GameInfo.Buildings() do
		if wonderClasses[row.BuildingClass] then
			wonders[row.ID] = true
		end
	end

	-- Scan players
	local teams = {}
	local players = {}
	local deathsCount = 0
	local wondersCount = 0
	local religionsCount = 0
	local naturalWondersCount = 0
	local nonNeutralCount = 0
	local hasSpies = false
	for playerID = 0, GameDefines.MAX_CIV_PLAYERS do
		local player = Players[playerID];
		if player and player:IsEverAlive() then
			players[playerID] = true

			if not (player:IsBarbarian()) then 
				teams[player:GetTeam()] = true
			else
				barbarianTeamID = player:GetTeam()
			end

			-- Deaths
			if (not player:IsAlive()) and not (player:IsBarbarian()) then 
				deathsCount = deathsCount + 1 
			end

			-- Wonders
			for city in player:Cities() do
				if city then
					for wonderID in pairs(wonders) do
						if city:GetNumRealBuilding(wonderID) > 0 then
							wondersCount = wondersCount + 1
						end
					end
				end
			end

			-- Natural wonders
			if GameDefines.HAPPINESS_PER_NATURAL_WONDER and GameDefines.HAPPINESS_PER_NATURAL_WONDER > 0 then
				naturalWondersCount = naturalWondersCount + (player:GetHappinessFromNaturalWonders() / GameDefines.HAPPINESS_PER_NATURAL_WONDER)
			end

			-- Pantheons & religions
			if player.HasCreatedPantheon and (player:HasCreatedPantheon() or player:HasCreatedReligion()) then
				religionsCount = religionsCount + 1
			end

			-- Num spies
			if player.GetNumSpies and player:GetNumSpies() > 0 then
				hasSpies = true
			end

			-- Non-neutral
			for otherPlayerID = 0, GameDefines.MAX_CIV_PLAYERS do
				if otherPlayerID > playerID and (player:GetApproachTowardsUsGuess(otherPlayerID) > 0)  and (not player:IsBarbarian()) and (not Players[otherPlayerID]:IsBarbarian()) then
					nonNeutralCount = nonNeutralCount + 1
				end
			end
		end
	end

	-- Scan teams
	local warsCount = 0
	local meetingsCount = 0
	local agreementsCount = 0
	local liberationsCount = 0
	for teamID in pairs(teams) do
		local team = Teams[teamID]
		for otherTeamID in pairs(teams) do
			if teamID < otherTeamID then
				local otherTeam = Teams[otherTeamID]

				if team:IsAtWar(otherTeamID) then
					warsCount = warsCount + 1
				end
				if team:IsHasMet(otherTeamID) then
					meetingsCount = meetingsCount + 1
				end
				if team:IsAllowsOpenBordersToTeam(otherTeamID) then
					agreementsCount = agreementsCount + 1
					print("open borders")
				end
				if team:IsDefensivePact(otherTeamID) then
					agreementsCount = agreementsCount + 1
					print("defensive pact")
				end
				if team:IsForcePeace(otherTeamID) then
					agreementsCount = agreementsCount + 1
					print("force peace")
				end
				if team:IsHasResearchAgreement(otherTeamID) then
					agreementsCount = agreementsCount + 1
					print("R&D")
				end
				if team:IsHasTradeAgreement(otherTeamID) then
					agreementsCount = agreementsCount + 1
					print("Trade agreement")
				end
				if team:GetLiberatedByTeam(otherTeamID) and team:IsMinorCiv() and not otherTeam:IsMinorCiv() then
					liberationsCount = liberationsCount + 1
					print("Liberated by", team:GetName(), Teams[otherTeamID]:GetName())
				end
			end
		end
	end

	-- Scan resources
	local archeoCount = 0
	if Map.GetPlotByIndex(0).GetArchaeologyArtifactType then
		for i = 0, Map.GetNumPlots() - 1, 1 do
			local plot = Map.GetPlotByIndex(i)
			if plot:GetArchaeologyArtifactType() ~= 0 then
				archeoCount = archeoCount + 1
			end
		end
	end


	local str = ""
	if deathsCount > 0 then
		str = str.."[NEWLINE][ICON_BULLET]"..L("TXT_KEY_RSD_UNRESETABLE_DEAD_CIVS", deathsCount)
	end
	if meetingsCount > 0 then
		str = str.."[NEWLINE][ICON_BULLET]"..L("TXT_KEY_RSD_UNRESETABLE_CONTACTS", meetingsCount)
	end
	if agreementsCount > 0 then
		str = str.."[NEWLINE][ICON_BULLET]"..L("TXT_KEY_RSD_UNRESETABLE_PACTS", agreementsCount)
	end
	if nonNeutralCount > 0 then
		str = str.."[NEWLINE][ICON_BULLET]"..L("TXT_KEY_RSD_UNRESETABLE_RELATIONSHIPS", nonNeutralCount)
	end
	if naturalWondersCount > 0 then
		str = str.."[NEWLINE][ICON_BULLET]"..L("TXT_KEY_RSD_UNRESETABLE_NATURAL_WONDERS", naturalWondersCount)
	end
	if wondersCount > 0 then
		str = str.."[NEWLINE][ICON_BULLET]"..L("TXT_KEY_RSD_UNRESETABLE_WONDERS", wondersCount)
	end
	if religionsCount > 0 then
		str = str.."[NEWLINE][ICON_BULLET]"..L("TXT_KEY_RSD_UNRESETABLE_RELIGIONS", religionsCount)
	end
	if liberationsCount > 0 then
		str = str.."[NEWLINE][ICON_BULLET]"..L("TXT_KEY_RSD_UNRESETABLE_LIBERATIONS", liberationsCount)
	end
	if warsCount > 0 then
		str = str.."[NEWLINE][ICON_BULLET]"..L("TXT_KEY_RSD_UNRESETABLE_WARS", warsCount)
	end
	if archeoCount > 0 then
		str = str.."[NEWLINE][ICON_BULLET]"..L("TXT_KEY_RSD_UNRESETABLE_ARCHEOLOGY_SITES", archeoCount)
	end
	if hasSpies then
		str = str.."[NEWLINE][ICON_BULLET]"..L("TXT_KEY_RSD_UNRESETABLE_SPIES")
	end
	
	return str	
end

-------------------------------------------------------------------------------------------------
function NextStep()
	if step == 0 then
		local str = GetUnresetableItems()
		if str ~= "" then
			Controls.Header:SetText("[COLOR_POSITIVE_TEXT]"..L("TXT_KEY_RSD_UNRESETABLE_HEADER").."[ENDCOLOR]")
			Controls.Message:SetText(L("TXT_KEY_RSD_UNRESETABLE_MESSAGE").."[NEWLINE]"..str)
			Controls.OK:SetText(L("TXT_KEY_RSD_UNRESETABLE_BUTTON"))
			Show()
			return
		end
		step = 1
	end

	if step == 1 then
		if (Game.GetNumCities() ~= 0) and (not CanDeleteCities()) then
			Controls.Header:SetText("[COLOR_POSITIVE_TEXT]"..L("TXT_KEY_RSD_RELAX_OPTIONS_HEADER").."[ENDCOLOR]")
			Controls.Message:SetText(L("TXT_KEY_RSD_RELAX_OPTIONS_MESSAGE"))
			Controls.OK:SetText(L("TXT_KEY_RSD_RELAX_OPTIONS_BUTTON"))
			Show()
			return
		end
		step = 2
	end

	if step == 2 then
		LuaEvents.RSD_Close()
		LuaEvents.RSD_ShowNewMapProgress()
		LuaEvents.RSD_BeginRoutine(mapRoutine)
		Events.SystemUpdateUI.CallImmediate(SystemUpdateUIType.BulkHideUI);
		OnCancel()
	end
end

-------------------------------------------------------------------------------------------------
function Show()
	if not shown then
		UIManager:QueuePopup(ContextPtr, PopupPriority.eUtmost);
		ContextPtr:SetHide(false);
	end
	Controls.MainStack:CalculateSize();
	Controls.MainGrid:DoAutoSize();
	shown = true
end

-------------------------------------------------------------------------------------------------
function OnCancel()
	if shown then
		ContextPtr:SetHide(true);
		UIManager:DequeuePopup(ContextPtr);
		mapRoutine = nil
		shown = false
	end
end
Controls.Cancel:RegisterCallback(Mouse.eLClick, OnCancel);

-------------------------------------------------------------------------------------------------
function OnOK()
	if step == 1 then
		RelaxOptions()
	end

	step = step + 1
	NextStep()
end
Controls.OK:RegisterCallback(Mouse.eLClick, OnOK);

-------------------------------------------------------------------------------------------------
function OnPopup(_mapRoutine)
	mapRoutine = _mapRoutine;
	shown = false
	step = 0
	NextStep()
end
LuaEvents.RSD_PromptRestartConfirmation.Add(OnPopup);

-------------------------------------------------------------------------------------------------
function OnInput(uiMsg, wParam, lParam)
	if uiMsg == KeyEvents.KeyDown and not ContextPtr:IsHidden() then
        if wParam == Keys.VK_ESCAPE then
			OnCancel();
            return true;
        elseif wParam == Keys.VK_RETURN then
			OnOK();
            return true;
        end
    end
end
ContextPtr:SetInputHandler(OnInput);

