-- Released under GPL v3
--------------------------------------------------------------
include("RSD_Utils")
local hasError = false
local STATE_HIDDEN = 0
local STATE_RUNNING = 1
local STATE_SUSPENDED = 2
local STATE_ERROR = 3
local currentState = STATE_HIDDEN

function SetState(state)
	hasError = state == STATE_ERROR
	ContextPtr:SetHide(state == STATE_HIDDEN)
	Controls.CloseButton:SetHide(state ~= STATE_ERROR)
	Controls.SuspendButton:SetHide(state == STATE_ERROR)
	if state == STATE_SUSPENDED then
		Controls.SuspendButtonLabel:SetText(L("TXT_KEY_RSD_RESUME"))
	else
		Controls.SuspendButtonLabel:SetText(L("TXT_KEY_RSD_SUSPEND"))
	end
	Controls.Stack:CalculateSize()
	Controls.Stack:ReprocessAnchoring()
	currentState = state
end

--------------------------------------------------------------
function OnShowNewMapProgress()
	Controls.ErrorHeader:SetText("")
	Controls.ErrorContent:SetText(L("TXT_KEY_RSD_RESTART_WARNING"))
	Controls.Status:SetText("[COLOR_POSITIVE_TEXT]"..L("TXT_KEY_RSD_STEP_WAIT").."[ENDCOLOR]")
	SetState(STATE_RUNNING)
end
LuaEvents.RSD_ShowNewMapProgress.Add(OnShowNewMapProgress)

--------------------------------------------------------------
function OnNewMapError(err)
	hasError = true
	local header = ""

	if err == "LOAD_MAP_ERROR" then
		local script = Locale.ToLower(PreGame.GetMapScript())
		local isDLC = string.find(script, "dlc_sp_maps", 1, true);		
		err = isDLC and L("TXT_KEY_RSD_DLC_MAP_LOADING_ERROR_CONTENT") or L("TXT_KEY_RSD_RESTART_MAP_ERROR_CONTENT")
		header = L("TXT_KEY_RSD_RESTART_MAP_ERROR_HEADER")

	elseif err == "MOVE_ERROR" then
		err = L("TXT_KEY_RSD_RESTART_MOVE_ERROR_CONTENT")
		header = L("TXT_KEY_RSD_RESTART_MOVE_ERROR_HEADER")

	else
		err = L("TXT_KEY_RSD_RESTART_ERROR_CONTENT").." "..err;
		header = L("TXT_KEY_RSD_RESTART_ERROR_HEADER")
	end

	Controls.ErrorHeader:SetText("[COLOR_YELLOW]"..header.."[ENDCOLOR]")
	Controls.ErrorContent:SetText(err)
	SetState(STATE_ERROR)
end
LuaEvents.RSD_RoutineError.Add(OnNewMapError)

--------------------------------------------------------------
function OnNewMapStatus(str)
	Controls.Status:SetText("[COLOR_POSITIVE_TEXT]"..str.."[ENDCOLOR]")
	SetState(STATE_RUNNING)
end
LuaEvents.RSD_NewMapStatus.Add(OnNewMapStatus)

--------------------------------------------------------------
function OnClose()
	SetState(STATE_HIDDEN)
	LuaEvents.RSD_Close()
end
Controls.CloseButton:RegisterCallback(Mouse.eLClick, OnClose);

--------------------------------------------------------------
function OnSuspendResume()
	if currentState == STATE_RUNNING then
		SetState(STATE_SUSPENDED)
		LuaEvents.RSD_SuspendRoutine()
	else
		SetState(STATE_RUNNING)
		LuaEvents.RSD_ResumeRoutine()
	end
end
Controls.SuspendButton:RegisterCallback(Mouse.eLClick, OnSuspendResume);

-------------------------------------------------------------------------------------------------
function OnInput(uiMsg, wParam, lParam)
	if hasError and uiMsg == KeyEvents.KeyDown and not ContextPtr:IsHidden() then
        if wParam == Keys.VK_ESCAPE or wParam == Keys.VK_RETURN then
			OnClose();
            return true;
        end
    end
end
ContextPtr:SetInputHandler(OnInput);

