-- Released under GPL v3
--------------------------------------------------------------
include("RSD_Utils");
include("IconSupport");
include("InstanceManager");
include("PlotMouseoverInclude");
include("ResourceTooltipGenerator");

local yieldManager = InstanceManager:new("OverlayYieldInstance", "Anchor", Controls.OverlayYieldContainer);
local yieldImageManager = InstanceManager:new("OverlayYieldImageInstance", "Image", Controls.OverlayYieldImageScrap);
local resourceManager = InstanceManager:new("OverlayResourceInstance", "Anchor", Controls.OverlayResourceContainer);
local revealMap = false


local resourceInstances = {};
local plotInstances = {};

local cultureTexture = "YieldAtlas_128_Culture.dds";
local defaultTexture = "YieldAtlas.dds";

local isGodsAndKings = true;
if Map.GetPlot(0,0).GetCulture then
	isGodsAndKings = false;
end

local options = {};
local useRsdOptions = false
LuaEvents.RSD_ShareOptions.Add(function(_options) options = _options end)

-------------------------------------------------------------------------------------------------
local resourcesVisibility = {}
local function UpdateResourceVisibility()
	local player = Players[Game.GetActivePlayer()];
	local team = Teams[player:GetTeam()];

	for row in GameInfo.Resources() do
		if options.showUnknownResources or (not row.TechReveal) then
			resourcesVisibility[row.ID] = true
		else
			local tech = GameInfoTypes[row.TechReveal];
			resourcesVisibility[row.ID] = team:GetTeamTechs():HasTech(tech);
		end
	end
end

--=======================================================================228========================
-- PLOT TOOLTIP
--===============================================================================================
local tipControls = {};
TTManager:GetTypeControlTable( "HexDetails", tipControls );

local function GetResourceString(plot)
	if plot:GetResourceType() < 0 then return "" end
	if not resourcesVisibility[plot:GetResourceType()] then return "" end

	local improvementStr = "";
	local resourceType = plot:GetResourceType();
	local pResource = GameInfo.Resources[resourceType];
		
	if plot:GetNumResource() > 1 then
		improvementStr = improvementStr .. plot:GetNumResource() .. " ";
	end
		
	local convertedKey = L(pResource.Description);		
	improvementStr = improvementStr .. pResource.IconString .. " " .. convertedKey;
		
	-- Resource Hookup info
	local iTechCityTrade = GameInfoTypes[pResource.TechCityTrade];
	if iTechCityTrade ~= nil and iTechCityTrade ~= -1 then
		local iActiveTeam = Game.GetActiveTeam();
		local pTeam = Teams[iActiveTeam];

		if not pTeam:GetTeamTechs():HasTech(iTechCityTrade) then
			local techName = GameInfo.Technologies[iTechCityTrade].Description;
			improvementStr = improvementStr .. " " .. L( "TXT_KEY_PLOTROLL_REQUIRES_TECH_TO_USE", techName );
		end
	end
	
	return improvementStr;
end

-------------------------------------------------------------------------------------------------
local function GetToolTipText(plot)
	if not plot then return "" end
	local str = "";

	-- Under construction
	if plot.GetBuildTurnsLeft then
		for pBuildInfo in GameInfo.Builds() do
			local iTurnsLeft = plot:GetBuildTurnsLeft(pBuildInfo.ID, 0, 0);
			if iTurnsLeft < 4000 and iTurnsLeft > 0 then
				if str ~= "" then str = str.."[NEWLINE]" end
				local convertedKey = L(pBuildInfo.Description);
				str = str..L("TXT_KEY_WORKER_BUILD_PROGRESS", iTurnsLeft, convertedKey);
			end
		end
	end

	-- City/plot owner
	local strOwner = GetOwnerString(plot);
	if strOwner ~= "" then
		if str ~= "" then str = str.."[NEWLINE]" end
		str = str..strOwner;
	end
		
	-- Resource
	local strResource = GetResourceString(plot, true);
	if strResource ~= "" then
		if str ~= "" then str = str.."[NEWLINE]" end
		str = str.."[COLOR_POSITIVE_TEXT]" .. L("TXT_KEY_RESOURCE") .. "[ENDCOLOR]" .. " : " .. strResource;
	end	
		
	-- Improvement & route
	local strImprovement = GetImprovementString(plot);
	if strImprovement ~= "" or plot:IsTradeRoute() then
		if str ~= "" then str = str.."[NEWLINE]" end

		local strTradeRouteBlock = "";
		if strImprovement ~= "" then
			str = str.."[COLOR_POSITIVE_TEXT]" .. L("TXT_KEY_IMPROVEMENT") .. "[ENDCOLOR]" .. " : " .. strImprovement;
			strTradeRouteBlock = ", ";
		end
		
		-- Trade Route
		if plot:IsTradeRoute() then
			str = str..strTradeRouteBlock..L( "TXT_KEY_PLOTROLL_TRADE_ROUTE" );			
		end
	end	
		
	-- Terrain type, feature
	local natureStr = GetNatureString(plot);
	if natureStr ~= "" then
		if str ~= "" then str = str.."[NEWLINE]" end
		str = str.."[COLOR_POSITIVE_TEXT]" .. L("TXT_KEY_PEDIA_TERRAIN_LABEL") .. "[ENDCOLOR]" .. " : " .. natureStr;
	end
		
	-- Yield
	local strYield = GetYieldString(plot);
	if strYield ~= "" then
		if str ~= "" then str = str.."[NEWLINE]" end
		str = str.."[COLOR_POSITIVE_TEXT]" .. L("TXT_KEY_OUTPUT") .. "[ENDCOLOR]" .. " : " .. strYield;
	end
		
	-- Presence of fresh water
	if plot:IsFreshWater() then
		if str ~= "" then str = str.."[NEWLINE]" end
		str = str..L("TXT_KEY_PLOTROLL_FRESH_WATER");
	end
		
	-- City state quest
	local cityStateStr = GetCivStateQuestString(plot, false);
	if cityStateStr ~= "" then
		if str ~= "" then str = str.."[NEWLINE]" end
		str = str..cityStateStr;
	end

	-- Coords
	if str ~= "" then str = str.."[NEWLINE]" end
	str = str..plot:GetX()..", "..plot:GetY()
	
	return str;
end

-------------------------------------------------------------------------------------------------
local function UpdateToolTipCore(plot)
	local str = GetToolTipText(plot);

	if str ~= "" then
		tipControls.HexDetailsText:SetText(str);
		tipControls.HexDetailsGrid:DoAutoSize();
		Controls.OverlayToolTipContainer:SetToolTipType("HexDetails");
	else
		Controls.OverlayToolTipContainer:SetToolTipType();
	end
end

-------------------------------------------------------------------------------------------------
function OnMouseOver( hexX, hexY )
	local plot = Map.GetPlot( hexX, hexY );
	if plot and (revealMap or plot:IsRevealed(Game.GetActiveTeam())) then 
		UpdateToolTipCore(plot) 
	end
end


--===============================================================================================
-- YIELD AND RESOURCES OVERLAY
--===============================================================================================
local function GetNumberOffset( number )
	number =  (number > 12) and 12 or number;
    local y = (number > 9) and 768 or 640;
    local x = 128 * ((number - 6) % 4);
    return x, y;
end

-------------------------------------------------------------------------------------------------
local function AppendYieldIcon(yieldType, amount, container, records)
    if amount <= 0 then return end

    local imageInstance = yieldImageManager:GetInstance();
	imageInstance.Image:ChangeParent(container);
	table.insert(records, imageInstance);
        
	-- Food, prod, gold, science, faith
    if( yieldType ~= YieldTypes.YIELD_CULTURE ) then
        imageInstance.Image:SetTexture(defaultTexture);
        if( amount >= 6 ) then
            imageInstance.Image:SetTextureOffsetVal( yieldType * 128, 512 );
        else
            imageInstance.Image:SetTextureOffsetVal( yieldType * 128, 128 * ( amount - 1 ) );
        end

		-- Additional image for number ?
		if( amount > 5 ) then
			local textImageInstance = yieldImageManager:GetInstance();
			textImageInstance.Image:SetTextureOffsetVal( GetNumberOffset( amount ) );
			textImageInstance.Image:ChangeParent( imageInstance.Image );
			table.insert(records, textImageInstance);
		end

	-- Culture
	else
        imageInstance.Image:SetTexture(cultureTexture);
        if( amount >= 5 ) then
            imageInstance.Image:SetTextureOffsetVal( 0 * 128, 512 );
        else
            imageInstance.Image:SetTextureOffsetVal( 0 * 128, 128 * ( amount - 1 ) );
        end
    end
end

-------------------------------------------------------------------------------------------------
local function UpdateYieldOverlay(plot, index, x, y)
	local visible = revealMap or plot:IsRevealed(Game.GetActiveTeam())
	local plotInstance = plotInstances[index];

	if not visible then
		if plotInstance then
			-- Clean sub-instances
			for _, v in pairs(plotInstance.images) do
				yieldImageManager:ReleaseInstance(v)
			end
			yieldManager:ReleaseInstance(plotInstance)
			plotInstances[index] = nil
		end
		return
	end

	if plotInstance then
		-- Clean sub-instances
		for _, v in pairs(plotInstance.images) do
			yieldImageManager:ReleaseInstance(v)
		end
	else
		-- Create instance
		plotInstance = yieldManager:GetInstance();
		plotInstances[index] = plotInstance;
	end

	-- Set it up
	plotInstance.images = {};
	AppendYieldIcon(0, plot:CalculateYield(0, true), plotInstance.Stack, plotInstance.images);
	AppendYieldIcon(1, plot:CalculateYield(1, true), plotInstance.Stack, plotInstance.images);
	AppendYieldIcon(2, plot:CalculateYield(2, true), plotInstance.Stack, plotInstance.images);
	AppendYieldIcon(3, plot:CalculateYield(3, true), plotInstance.Stack, plotInstance.images);
	if isGodsAndKings then
		AppendYieldIcon(4, plot:CalculateYield(4, true), plotInstance.CultureContainer, plotInstance.images);
		AppendYieldIcon(5, plot:CalculateYield(5, true), plotInstance.CultureContainer, plotInstance.images);
	else
		AppendYieldIcon(4, plot:GetCulture(), plotInstance.CultureContainer, plotInstance.images);
	end
	plotInstance.Stack:CalculateSize();
	plotInstance.Stack:ReprocessAnchoring();
	plotInstance.Anchor:SetHexPosition(x, y);
end

-------------------------------------------------------------------------------------------------
local function UpdateResourceOverlay(plot, index, x, y)
	local visible = revealMap or plot:IsRevealed(Game.GetActiveTeam())

	-- No resource or not visible? Delete old instance
	local resourceType = plot:GetResourceType();
	if (not resourceType) or (resourceType == -1) or (not resourcesVisibility[resourceType]) or (not visible) then
		local instance = resourceInstances[index];
		if instance then resourceManager:ReleaseInstance(instance) end
		return;
	end
	
	-- Create/retrieve instance
	local instance = resourceInstances[index] or resourceManager:GetInstance();
	resourceInstances[index] = instance;

	-- Update instance
	local resourceInfo = GameInfo.Resources[resourceType]
	IconHookup(resourceInfo.PortraitIndex, 64, resourceInfo.IconAtlas, instance.ResourceIcon);

	local strToolTip = GenerateResourceToolTip(plot);
	instance.ResourceIcon:SetToolTipString(strToolTip or "");

	-- Position instance
	local wx, wy, wz = GridToWorld(x, y);
	instance.Anchor:SetWorldPositionVal(wx - 30, wy + 15, wz);
end

-------------------------------------------------------------------------------------------------
local function Update()
	if ContextPtr:IsHidden() then return end
	OptionsManager.SyncGameOptionsCache()

	local showYields, showResources
	if useRsdOptions then
		showYields = options.showYields
		showResources = false;
	else
		showYields = OptionsManager.GetYieldOn();
		showResources = OptionsManager.GetResourceOn() and not InStrategicView();
	end
	print("show yields", showYields)

    Controls.OverlayYieldContainer:SetHide(not showYields);
    Controls.OverlayResourceContainer:SetHide(not showResources);

	if showYields then
		yieldImageManager:ResetInstances();
		yieldManager:ResetInstances();
		plotInstances = {};

		for i = 0, Map.GetNumPlots()-1, 1 do
			local plot = Map.GetPlotByIndex(i);
			UpdateYieldOverlay(plot, i, plot:GetX(), plot:GetY());
		end
	end

	if showResources then
		UpdateResourceVisibility();
		resourceManager:ResetInstances();
		resourceInstances = {};

		for i = 0, Map.GetNumPlots()-1, 1 do
			local plot = Map.GetPlotByIndex(i);
			UpdateResourceOverlay(plot, i, plot:GetX(), plot:GetY());
		end
	end
end
LuaEvents.RSD_UpdateResources.Add(Update)
LuaEvents.RSD_UpdateYields.Add(Update)
Events.StrategicViewStateChanged.Add(Update)
Events.GameOptionsChanged.Add(Update);

-------------------------------------------------------------------------------------------------
local function OnShowOverlay()
	LookUpControl("InGame/ResourceIconManager"):SetHide(true);
	LookUpControl("InGame/PlotHelpManager"):SetHide(true);
	LookUpControl("InGame/YieldIconManager"):SetHide(true);
	ContextPtr:SetHide(false)
	Update()

	Events.SerialEventMouseOverHex.Add(OnMouseOver);
end
LuaEvents.RSD_ShowOverlay.Add(OnShowOverlay)

-------------------------------------------------------------------------------------------------
local function OnHideOverlay()
	if ContextPtr:IsHidden() then return end

	Events.SerialEventMouseOverHex.Remove(OnMouseOver);
	ContextPtr:SetHide(true)

	LookUpControl("InGame/ResourceIconManager"):SetHide(false);
	LookUpControl("InGame/YieldIconManager"):SetHide(false);
	LookUpControl("InGame/PlotHelpManager"):SetHide(false);

	--OptionsManager.SyncGameOptionsCache()
	--OptionsManager.Set
	--OptionsManager.CommitGameOptions(PreGame.IsHotSeatGame());
end
LuaEvents.RSD_HideOverlay.Add(OnHideOverlay)

-------------------------------------------------------------------------------------------------
local defaultMap = Map
local function OnShowMap(_map)
	print("RESEED: StrategicOverlay.OnShowPreview")
	Map = _map
	useRsdOptions = true
	Update()
end
LuaEvents.RSD_ShowPreview.Add(OnShowMap)

-------------------------------------------------------------------------------------------------
local function OnHideMap()
	if Map == defaultMap and (not useRsdOptions) then return end

	Map = defaultMap
	useRsdOptions = false
	if not ContextPtr:IsHidden() then
		Update()
	end
end
LuaEvents.RSD_HidePreview.Add(OnHideMap)


-------------------------------------------------------------------------------------------------
local function OnRevealMapChanged(_revealMap)
	revealMap = _revealMap
	Update()
end
LuaEvents.RSD_RevealMapChanged.Add(OnRevealMapChanged)