-- Released under GPL v3
--------------------------------------------------------------
print("'Reseed!'... loading")
include("InstanceManager");
include("RSD_Utils");
include("RSD_Overrides");
include("RSD_Process");

local earthMaps = {}
earthMaps["Assets\\Maps\\Earth_Duel.Civ5Map"] = true
earthMaps["Assets\\Maps\\Earth_Tiny.Civ5Map"] = true
earthMaps["Assets\\Maps\\Earth_Small.Civ5Map"] = true
earthMaps["Assets\\Maps\\Earth_Standard.Civ5Map"] = true
earthMaps["Assets\\Maps\\Earth_Large.Civ5Map"] = true
earthMaps["Assets\\Maps\\Earth_Huge.Civ5Map"] = true

local optionsManager = InstanceManager:new("DropDownOptionInstance", "DropDownOptionRoot", Controls.MapOptionsStack);
local initialized = false;
local isPreviewing = false;
local isOverlayVisible = false;
local mapToGenerate = nil;
local selectedMap = nil;
local revealMap = false;

local options = {};
LuaEvents.RSD_ShareOptions.Add(function(_options) options = _options end)

-------------------------------------------------------------------------------------------------
function OnHiddenFog()
	Controls.ReloadButton:SetHide(false)
	UpdateLayout()
end
LuaEvents.RSD_HiddenFog.Add(OnHiddenFog)

-------------------------------------------------------------------------------------------------
function OnRevealMapChanged(revealed)
	if revealed then
		Controls.RevealMapButton:SetText(L("TXT_KEY_RSD_HIDE_FOG"));
	else
		Controls.RevealMapButton:SetText(L("TXT_KEY_RSD_SHOW_FOG"));
	end
	UpdateLayout()
end
LuaEvents.RSD_RevealMapChanged.Add(OnRestoredFog)

-------------------------------------------------------------------------------------------------
local function OnToggleRevealMap()
	LuaEvents.RSD_ToggleRevealMap()
end
Controls.RevealMapButton:RegisterCallback(Mouse.eLClick, OnToggleRevealMap);

-------------------------------------------------------------------------------------------------
local function OnSaveAndReload()
	LuaEvents.RSD_Close()

	-- Save and reload
	--print("Deleting previous file");
	--DeleteFiles("__reseed");
	print("Saving");
	UI.SaveGame("__reseed", true);
	print("Reloading");
	LoadFile("__reseed");
end
Controls.ReloadButton:RegisterCallback(Mouse.eLClick, OnSaveAndReload);

-------------------------------------------------------------------------------------------------
local function OnRestartNewMap()
	local newMapCoroutine = CreateNewMapCoroutine(mapToGenerate, selectedMap.current)
	LuaEvents.RSD_PromptRestartConfirmation(newMapCoroutine)
end
Controls.RestartNewMapButton:RegisterCallback(Mouse.eLClick, OnRestartNewMap);

-------------------------------------------------------------------------------------------------
local function OnPreviewNewMap()
	isPreviewing = true
	local coroutineInvoker = CreateCoroutine(function()
		mapToGenerate = GenerateRsdMap()
		print("RESEED: ShowPreview")
		LuaEvents.RSD_ShowPreview(mapToGenerate, mapToGenerate.GetStartingPlot)

		-- Move camera
		RsdMap = mapToGenerate
		local rsdPlot = mapToGenerate.GetStartingPlot(Players[Game.GetActivePlayer()])
		local plot = Map.GetPlot(rsdPlot:GetX(), rsdPlot:GetY())
		UI.LookAt(plot, 0)
	end)
	LuaEvents.RSD_BeginRoutine(coroutineInvoker)
end
Controls.PreviewNewMapButton:RegisterCallback(Mouse.eLClick, OnPreviewNewMap);
LuaEvents.RSD_PreviewNewMap.Add(OnPreviewNewMap)

-------------------------------------------------------------------------------------------------
local function OnShowUnknownResources()
	options.showUnknownResources = not options.showUnknownResources
	LuaEvents.RSD_UpdateResources()
	LuaEvents.RSD_SaveOptions()
end
Controls.ShowUnknownResources:RegisterCheckHandler(OnShowUnknownResources);

-------------------------------------------------------------------------------------------------
local function OnShowImprovements()
	options.showImprovements = not options.showImprovements
	LuaEvents.RSD_UpdateResources()
	LuaEvents.RSD_SaveOptions()
end
Controls.ShowImprovements:RegisterCheckHandler(OnShowImprovements);

-------------------------------------------------------------------------------------------------
local function OnShowYields()
	options.showYields = not options.showYields
	LuaEvents.RSD_UpdateYields()
	LuaEvents.RSD_SaveOptions()
end
Controls.ShowYields:RegisterCheckHandler(OnShowYields);

-------------------------------------------------------------------------------------------------
local function OnHideMap()
	isPreviewing = false
	UpdateLayout()
end
LuaEvents.RSD_HidePreview.Add(OnHideMap)

-------------------------------------------------------------------------------------------------
local function OnClose()
    ContextPtr:SetHide(true)
	mapToGenerate = nil
end
LuaEvents.RSD_Close.Add(OnClose)

-------------------------------------------------------------------------------------------------
local function OnCloseButton()
	LuaEvents.RSD_Close()
end
Controls.CloseButton:RegisterCallback(Mouse.eLClick, OnCloseButton);

-------------------------------------------------------------------------------------------------
local function OnMinimize()
	Controls.MainPanel:SetHide(true)
	Controls.MiniPanel:SetHide(false)
end
Controls.MinimizeButton:RegisterCallback(Mouse.eLClick, OnMinimize);

-------------------------------------------------------------------------------------------------
local function OnRestoreButton()
	Controls.MainPanel:SetHide(false)
	Controls.MiniPanel:SetHide(true)
end
Controls.RestoreButton:RegisterCallback(Mouse.eLClick, OnRestoreButton);

-------------------------------------------------------------------------------------------------
local function OnFixDLC()
	LuaEvents.RSD_ShowDLCFix()
end
Controls.FixDLCButton:RegisterCallback(Mouse.eLClick, OnFixDLC)

-------------------------------------------------------------------------------------------------
local function OnShowOverlay()
	isOverlayVisible = true
	UpdateLayout()
end
LuaEvents.RSD_ShowOverlay.Add(OnShowOverlay)

-------------------------------------------------------------------------------------------------
local function OnHideOverlay()
	isOverlayVisible = false
	UpdateLayout()
end
LuaEvents.RSD_HideOverlay.Add(OnHideOverlay)

-------------------------------------------------------------------------------------------------
local function OnShow()
	if not initialized then
		print("initialize");
		xpcall(Initialize, OnInitializationError);
		print("success");
	end
    ContextPtr:SetHide(false)
end
LuaEvents.RSD_Show.Add(OnShow)

-------------------------------------------------------------------------------------------------
local function GetScriptError(script, mapData)
	if mapData then
		return "RSD_STATIC_MAP"
	end

	script = Locale.ToLower(script)
	if string.find(script, "dlc_sp_maps", 1, true) then
		return "RSD_DLC_MAP"
	end
end

-------------------------------------------------------------------------------------------------
local mapScripts = {};
local mapOptions = {};
function InitializeMapScripts()
	print("InitializeMapScripts");

	-- Earth entry
	local earthEntry = {
		earth = true,
		error = "RSD_STATIC_MAP",
		name = L("TXT_KEY_MAP_EARTH_TITLE"),
		description = L("TXT_KEY_MAP_EARTH_HELP"),
	}
	table.insert(mapScripts, earthEntry);	

	-- Scan scripts
	for row in GameInfo.MapScripts{SupportsSinglePlayer = 1} do
		local entry = {
			file = row.FileName,
			name = Locale.ConvertTextKey(row.Name),
			description = (row.Description and Locale.ConvertTextKey(row.Description) or ""),
			error = GetScriptError(row.FileName)
		}

		if not earthMaps[entry.file] then
			table.insert(mapScripts, entry);
			--print(entry.name, tostring(entry.error))
		end
	end	

	-- Scan map files		
	for _, map in ipairs(Modding.GetMapFiles()) do
		local mapData = UI.GetMapPreview(map.File);
		local entry = { file = map.File }

		-- Name
		if not Locale.IsNilOrWhitespace(map.Name) then
			entry.name = map.Name;
		elseif mapData and (not Locale.IsNilOrWhitespace(mapData.Name)) then
			entry.name = Locale.Lookup(mapData.Name);
		else
			entry.name = Path.GetFileNameWithoutExtension(map.File);
		end

		-- Description
		if(map.Description and #map.Description > 0) then
			entry.description = map.Description;
		elseif mapData then
			entry.description = Locale.ConvertTextKey(mapData.Description);
		end

		-- Error
		entry.error = GetScriptError(map.File, mapData)

		-- Insert
		if not earthMaps[entry.file] then
			table.insert(mapScripts, entry);
			--print(entry.name, tostring(entry.error))
		end
	end

	-- Special errors flags
	for _, map in ipairs(mapScripts) do
		map.static = (map.error == "RSD_STATIC_MAP")
		map.dlc = (map.error == "RSD_DLC_MAP")
	end

	-- "Current map" entry
	local currentMapEntry = {
		current = true,
		name = L("TXT_KEY_RSD_CURRENT_MAP"),
		description = L("TXT_KEY_RSD_CURRENT_MAP_HINT"),
	};
	table.sort(mapScripts, function(a, b) return a.name < b.name end);
	table.insert(mapScripts, 1, currentMapEntry);	

	-- Find selected map
	selectedMap = PreGame.IsEarthMap() and earthEntry or currentMapEntry
	local pregameFile = PreGame.GetMapScript();
	for _, map in ipairs(mapScripts) do
		if (map.file == pregameFile) and (not map.isStatic) then
			selectedMap = map
			break
		end
	end
end

-------------------------------------------------------------------------------------------------
function InitializeMapOptions()
	print("InitializeMapOptions");
	mapOptions = {};
	if selectedMap.current then return end
	local pregameFile = PreGame.GetMapScript();
	
	-- Query options
	for option in DB.Query("select * from MapScriptOptions where exists (select 1 from MapScriptOptionPossibleValues where FileName = MapScriptOptions.FileName and OptionID = MapScriptOptions.OptionID) and Hidden = 0 and FileName = ?", pregameFile) do
		mapOptions[option.OptionID] = {
			ID = option.OptionID,
			name = Locale.ConvertTextKey(option.Name),
			disabled = (option.ReadOnly == 1) and true or false,
			description = (option.Description) and Locale.ConvertTextKey(option.Description) or nil,
			defaultValue = option.DefaultValue,
			sortPriority = option.SortPriority,
			values = {},
		}; 
	end
	
	-- Query values for every option
	for possibleValue in DB.Query("select * from MapScriptOptionPossibleValues where FileName = ? order by SortIndex ASC", pregameFile) do
		if(mapOptions[possibleValue.OptionID] ~= nil) then
			table.insert(mapOptions[possibleValue.OptionID].values, 
			{
				description = (possibleValue.Description) and Locale.ConvertTextKey(possibleValue.Description) or nil,
				name	= Locale.ConvertTextKey(possibleValue.Name),
				value	= possibleValue.Value,
			});
		end
	end
end

-------------------------------------------------------------------------------------------------
function OnInitializationError(err)
	if reportedError then return end
	reportedError = true;
	print(err);

	print("Failed to initialize 'Reseed!'");
	err = FormatError(err, 3);

	-- Show popup to user
	local str = L("TXT_KEY_RSD_INITIALIZATION_ERROR").."[NEWLINE][ICON_PIRATE] "..err;
	Events.SerialEventGameMessagePopup( { Type = ButtonPopupTypes.BUTTONPOPUP_TEXT, Data1 = 800, Option1 = true, Text = str } );
end

-------------------------------------------------------------------------------------------------
function Initialize()
	InitializeMapScripts();

	-- Fills up the map type pulldown
	Controls.MapTypePullDown:ClearEntries();
	for i, script in ipairs(mapScripts) do
		local instance = {};
		Controls.MapTypePullDown:BuildEntry("InstanceOne", instance);
		instance.Button:SetToolTipString(script.description);
		instance.Button:SetVoid1(i);

		if script.error then
			instance.Button:SetText("[ICON_LOCKED] "..script.name);
		else
			instance.Button:SetText("  "..script.name);
		end
	end
	Controls.MapTypePullDown:CalculateInternals();

	-- Fills up the speed pulldown
	for i = 1, 4, 1 do
		local controlTable = {};
		Controls.SpeedPullDown:BuildEntry("InstanceOne", controlTable);
		controlTable.Button:SetText(L("TXT_KEY_RSD_SPEED_"..i));
		controlTable.Button:SetVoid1(i);
	end
	Controls.SpeedPullDown:CalculateInternals();
	Controls.FakeRestartNewMapButton:SetDisabled(true)

	InitializeMapOptions();
	UpdateUI();
	initialized = true;
	print("Initialize - Done");
end

-------------------------------------------------------------------------------------------------
function UpdateOptionsUI()
	-- Sort options
	local sortedOptions = {};
	for _, option in pairs(mapOptions) do
		table.insert(sortedOptions, option);
	end
	table.sort(sortedOptions, function(a, b) 
		if a.sortPriority == b.sortPriority then
			return a.name < b.name;
		else
			return a.sortPriority < b.sortPriority;
		end
	end);

	-- Options combos
	print("Update options");
	optionsManager:ResetInstances();
	if selectedMap.current then return end

	for _, option in ipairs(sortedOptions) do
		local instance = optionsManager:GetInstance();
		instance.OptionName:SetText(option.name);
		instance.OptionName:SetToolTipString(option.description);
		instance.OptionDropDown:SetDisabled(option.disabled);
		local dropDownButton = instance.OptionDropDown:GetButton();
		
		-- Possible values for every option
		instance.OptionDropDown:ClearEntries();
		for _, possibleValue in ipairs(option.values) do
			subInstance = {};
			instance.OptionDropDown:BuildEntry("InstanceOne", subInstance);
			subInstance.Button:SetText(possibleValue.name);
			subInstance.Button:SetToolTipString(possibleValue.description or nil);
			
			-- Callback
			subInstance.Button:RegisterCallback(Mouse.eLClick, function()
				dropDownButton:SetText(possibleValue.name);
				if possibleValue.description then
					dropDownButton:SetToolTipString(possibleValue.description);
				else
					dropDownButton:SetToolTipString();
				end
				LuaEvents.RSD_HidePreview()
				PreGame.SetMapOption(option.ID, possibleValue.value);
				UpdateUI();
			end);
		end
		
		-- Get current value
		local value = option.values[PreGame.GetMapOption(option.ID)] or option.values[option.defaultValue];
		
		-- Set current value on drop down
		if value then
			dropDownButton:SetText(value.name or "");
			dropDownButton:SetToolTipString(value.description or "");
		else
			dropDownButton:SetText("");
			dropDownButton:SetToolTipString("");
		end
	
		dropDownButton:SetDisabled(option.disabled);
		instance.OptionDropDown:CalculateInternals();
	end
	print("Update Options - Done")
end

-------------------------------------------------------------------------------------------------
function UpdateLayout()
	print("Update layout - begin")
	if not selectedMap then return end
	local showResourcesWhenNotPreviewing = OptionsManager.GetResourceOn() and not InStrategicView()
	local canShowUnknownResourcesWhenNotPreviewing = showResourcesWhenNotPreviewing and isOverlayVisible

	Controls.ShowYields:SetHide(not isPreviewing)
	Controls.ShowYields:SetCheck(options.showYields);
	Controls.ShowImprovements:SetHide(not isPreviewing);
	Controls.ShowImprovements:SetCheck(options.showImprovements);
	Controls.ShowUnknownResources:SetCheck(options.showUnknownResources);
	Controls.ShowUnknownResources:SetHide((not isPreviewing) and (not canShowUnknownResourcesWhenNotPreviewing));

	print("Update restart label")
	if selectedMap.current then
		-- Set tool tip as "Restart (current map)"
		Controls.RestartNewMapLabel:SetText(L("TXT_KEY_RSD_RESTART_CURRENT_MAP"))
		Controls.RestartNewMapButton:SetToolTipString(L("TXT_KEY_RSD_RESTART_CURRENT_MAP_HELP"))
	elseif isPreviewing then
		-- Set tool tip as "Restart (previewed map)"
		Controls.RestartNewMapLabel:SetText(L("TXT_KEY_RSD_RESTART_PREVIEWED_MAP"))
		Controls.RestartNewMapButton:SetToolTipString(L("TXT_KEY_RSD_RESTART_PREVIEWED_MAP_HELP"))
	else
		-- Set tool tip as "Restart (new map)"
		Controls.RestartNewMapLabel:SetText(L("TXT_KEY_RSD_RESTART_NEW_MAP"))
		Controls.RestartNewMapButton:SetToolTipString(L("TXT_KEY_RSD_RESTART_NEW_MAP_HELP"))
	end

	-- Update layout
	print("Update stack sizes")
	Controls.MapOptionsStack:CalculateSize();
	Controls.MapOptionsStack:ReprocessAnchoring();
	Controls.MainStack:CalculateSize();
	Controls.MainStack:ReprocessAnchoring();
	Controls.Background:SetSizeX(Controls.MainStack:GetSizeX())
	Controls.Background:SetSizeY(Controls.MainStack:GetSizeY())
	Controls.MainPanel:SetSizeX(Controls.MainStack:GetSizeX() + 60);
	Controls.MainPanel:SetSizeY(Controls.MainStack:GetSizeY() + 100);
	Controls.MainPanel:ReprocessAnchoring();

	-- Change alignements of left-aligned items that need to align with the right column
	local offset = Controls.MainStack:GetSizeX() - 220
	Controls.MapErrorLabel:SetOffsetX(offset)
	Controls.ShowUnknownResources:SetOffsetX(offset)
	Controls.ShowImprovements:SetOffsetX(offset)
	Controls.ShowYields:SetOffsetX(offset)
	print("Update layout - done")
end
Events.StrategicViewStateChanged.Add(UpdateLayout)
Events.GameOptionsChanged.Add(UpdateLayout);

-------------------------------------------------------------------------------------------------
function UpdateUI()
	print("Map:",			tostring(selectedMap and selectedMap.name),		tostring(selectedMap and selectedMap.file));
	print("GetWorldSize:",	tostring(PreGame.GetWorldSize()));

	-- Speed pulldown
	Controls.SpeedPullDown:GetButton():LocalizeAndSetText("TXT_KEY_RSD_SPEED_"..options.speed);

	-- Map script pulldown
	print("Update maps");
	Controls.MapTypePullDown:GetButton():LocalizeAndSetText(selectedMap.name);
	Controls.MapTypePullDown:GetButton():LocalizeAndSetToolTip(selectedMap.description or "");
	
	-- Update options or report error
	local errorMsg = "";
	if selectedMap.current then
		errorMsg = L("TXT_KEY_RSD_CURRENT_MAP_HINT")
	elseif not selectedMap.error then
		UpdateOptionsUI()
	elseif selectedMap.static then
		errorMsg = L("TXT_KEY_RSD_STATIC_MAP_ERROR")
	elseif selectedMap.dlc then
		errorMsg = L("TXT_KEY_RSD_DLC_MAP_ERROR")
	else
		errorMsg = L("TXT_KEY_RSD_MAP_ERROR")
	end

	print("Update error status")
	local errorOrCurrentMap = selectedMap.error or selectedMap.current
	Controls.MapErrorLabel:SetText("[COLOR_YELLOW]"..errorMsg.."[ENDCOLOR]")
	Controls.PreviewNewMapButton:SetDisabled(errorOrCurrentMap)
	Controls.MapOptionsStack:SetHide(errorOrCurrentMap)
	Controls.MapErrorLabel:SetHide(not errorOrCurrentMap)
	Controls.FakeRestartNewMapButton:SetHide(not selectedMap.error)
	Controls.RestartNewMapButton:SetHide(selectedMap.error)
	Controls.FixDLCButton:SetHide(not selectedMap.dlc)

	UpdateLayout()
	print("Done update")
end

-------------------------------------------------------------------------------------------------
function OnMapScriptChanged(id)
	selectedMap = mapScripts[id];
	PreGame.SetEarthMap(selectedMap.earth);
	PreGame.SetRandomMapScript(selectedMap.random);
	PreGame.SetMapScript(selectedMap.file or "");
	LuaEvents.RSD_HidePreview()
	InitializeMapOptions();
	UpdateUI();
end
Controls.MapTypePullDown:RegisterCallback(Mouse.eLClick, OnMapScriptChanged);

-------------------------------------------------------------------------------------------------
function OnSpeedChanged(id)
	options.speed = id
	LuaEvents.RSD_SaveOptions()
	UpdateUI();
end
Controls.SpeedPullDown:RegisterCallback(Mouse.eLClick, OnSpeedChanged);

